#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['redash_stmo',
 'redash_stmo.data_sources',
 'redash_stmo.data_sources.link',
 'redash_stmo.data_sources.validator',
 'redash_stmo.data_sources.version',
 'redash_stmo.handlers',
 'redash_stmo.handlers.authentication',
 'redash_stmo.handlers.query_results',
 'redash_stmo.query_runner']

package_data = \
{'': ['*'],
 'redash_stmo.data_sources.link': ['bundle/*'],
 'redash_stmo.data_sources.validator': ['bundle/*'],
 'redash_stmo.data_sources.version': ['bundle/*']}

install_requires = \
['dockerflow>=2018.4.0', 'requests', 'sqlparse', 'six', 'pyhive', 'pgsanity']

extras_require = \
{'dev': ['watchdog'], 'test': ['pytest', 'mock', 'pytest-cov', 'pytest-flake8']}

entry_points = \
{'redash.extensions': ['datasource_health = '
                       'redash_stmo.data_sources.health:extension',
                       'datasource_link = '
                       'redash_stmo.data_sources.link:extension',
                       'datasource_validator = '
                       'redash_stmo.data_sources.validator:extension',
                       'datasource_version = '
                       'redash_stmo.data_sources.version:extension',
                       'dockerflow = redash_stmo.dockerflow:extension',
                       'handler_queryresults = '
                       'redash_stmo.handlers.query_results:extension',
                       'handler_remote_user_auth = '
                       'redash_stmo.handlers.authentication.remote_user_auth:extension',
                       'queryrunner_presto = '
                       'redash_stmo.query_runner.presto:extension']}

setup(name='redash-stmo',
      version='2019.3.1',
      description='Extensions to Redash by Mozilla',
      author='Mozilla Foundation',
      author_email='dev-webdev@lists.mozilla.org',
      url='https://github.com/mozilla/redash-stmo',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
     )
